# Retrieve the theme settings
export def main [] {
    return {
        binary: '#d05ec1'
        block: '#246eb2'
        cell-path: '#bfbfbf'
        closure: '#00acee'
        custom: '#e5e5e5'
        duration: '#bebf00'
        float: '#e50000'
        glob: '#e5e5e5'
        int: '#d05ec1'
        list: '#00acee'
        nothing: '#ef5734'
        range: '#bebf00'
        record: '#00acee'
        string: '#2baf2b'

        bool: {|| if $in { '#00e5e5' } else { '#bebf00' } }

        datetime: {|| (date now) - $in |
            if $in < 1hr {
                { fg: '#ef5734' attr: 'b' }
            } else if $in < 6hr {
                '#ef5734'
            } else if $in < 1day {
                '#bebf00'
            } else if $in < 3day {
                '#2baf2b'
            } else if $in < 1wk {
                { fg: '#2baf2b' attr: 'b' }
            } else if $in < 6wk {
                '#00acee'
            } else if $in < 52wk {
                '#246eb2'
            } else { 'dark_gray' }
        }

        filesize: {|e|
            if $e == 0b {
                '#bfbfbf'
            } else if $e < 1mb {
                '#00acee'
            } else {{ fg: '#246eb2' }}
        }

        shape_and: { fg: '#d05ec1' attr: 'b' }
        shape_binary: { fg: '#d05ec1' attr: 'b' }
        shape_block: { fg: '#246eb2' attr: 'b' }
        shape_bool: '#00e5e5'
        shape_closure: { fg: '#00acee' attr: 'b' }
        shape_custom: '#2baf2b'
        shape_datetime: { fg: '#00acee' attr: 'b' }
        shape_directory: '#00acee'
        shape_external: '#00acee'
        shape_external_resolved: '#00e5e5'
        shape_externalarg: { fg: '#2baf2b' attr: 'b' }
        shape_filepath: '#00acee'
        shape_flag: { fg: '#246eb2' attr: 'b' }
        shape_float: { fg: '#e50000' attr: 'b' }
        shape_garbage: { fg: '#FFFFFF' bg: '#FF0000' attr: 'b' }
        shape_glob_interpolation: { fg: '#00acee' attr: 'b' }
        shape_globpattern: { fg: '#00acee' attr: 'b' }
        shape_int: { fg: '#d05ec1' attr: 'b' }
        shape_internalcall: { fg: '#00acee' attr: 'b' }
        shape_keyword: { fg: '#d05ec1' attr: 'b' }
        shape_list: { fg: '#00acee' attr: 'b' }
        shape_literal: '#246eb2'
        shape_match_pattern: '#2baf2b'
        shape_matching_brackets: { attr: 'u' }
        shape_nothing: '#ef5734'
        shape_operator: '#bebf00'
        shape_or: { fg: '#d05ec1' attr: 'b' }
        shape_pipe: { fg: '#d05ec1' attr: 'b' }
        shape_range: { fg: '#bebf00' attr: 'b' }
        shape_raw_string: { fg: '#e5e5e5' attr: 'b' }
        shape_record: { fg: '#00acee' attr: 'b' }
        shape_redirection: { fg: '#d05ec1' attr: 'b' }
        shape_signature: { fg: '#2baf2b' attr: 'b' }
        shape_string: '#2baf2b'
        shape_string_interpolation: { fg: '#00acee' attr: 'b' }
        shape_table: { fg: '#246eb2' attr: 'b' }
        shape_vardecl: { fg: '#246eb2' attr: 'u' }
        shape_variable: '#d05ec1'

        foreground: '#ffcc2f'
        background: '#2c1d16'
        cursor: '#ffcc2f'

        empty: '#246eb2'
        header: { fg: '#2baf2b' attr: 'b' }
        hints: '#666666'
        leading_trailing_space_bg: { attr: 'n' }
        row_index: { fg: '#2baf2b' attr: 'b' }
        search_result: { fg: '#ef5734' bg: '#bfbfbf' }
        separator: '#bfbfbf'
    }
}

# Update the Nushell configuration
export def --env "set color_config" [] {
    $env.config.color_config = (main)
}

# Update terminal colors
export def "update terminal" [] {
    let theme = (main)

    # Set terminal colors
    let osc_screen_foreground_color = '10;'
    let osc_screen_background_color = '11;'
    let osc_cursor_color = '12;'
        
    $"
    (ansi -o $osc_screen_foreground_color)($theme.foreground)(char bel)
    (ansi -o $osc_screen_background_color)($theme.background)(char bel)
    (ansi -o $osc_cursor_color)($theme.cursor)(char bel)
    "
    # Line breaks above are just for source readability
    # but create extra whitespace when activating. Collapse
    # to one line and print with no-newline
    | str replace --all "\n" ''
    | print -n $"($in)\r"
}

export module activate {
    export-env {
        set color_config
        update terminal
    }
}

# Activate the theme when sourced
use activate